
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

            #pragma once

            #include <glib-object.h>

#if !defined(ST_H_INSIDE) && !defined(ST_COMPILATION)
#error "Only <st/st.h> can be included directly.h"
#endif

            G_BEGIN_DECLS

/* enumerations from "st-button.h" */

GType st_button_mask_get_type (void);
#define ST_TYPE_BUTTON_MASK (st_button_mask_get_type())

/* enumerations from "st-clipboard.h" */

GType st_clipboard_type_get_type (void);
#define ST_TYPE_CLIPBOARD_TYPE (st_clipboard_type_get_type())

/* enumerations from "st-icon-theme.h" */

GType st_icon_lookup_flags_get_type (void);
#define ST_TYPE_ICON_LOOKUP_FLAGS (st_icon_lookup_flags_get_type())

GType st_icon_theme_error_get_type (void);
#define ST_TYPE_ICON_THEME_ERROR (st_icon_theme_error_get_type())

/* enumerations from "st-scroll-view.h" */

GType st_policy_type_get_type (void);
#define ST_TYPE_POLICY_TYPE (st_policy_type_get_type())

/* enumerations from "st-settings.h" */

GType st_system_color_scheme_get_type (void);
#define ST_TYPE_SYSTEM_COLOR_SCHEME (st_system_color_scheme_get_type())

GType st_system_accent_color_get_type (void);
#define ST_TYPE_SYSTEM_ACCENT_COLOR (st_system_accent_color_get_type())

/* enumerations from "st-texture-cache.h" */

GType st_texture_cache_policy_get_type (void);
#define ST_TYPE_TEXTURE_CACHE_POLICY (st_texture_cache_policy_get_type())

/* enumerations from "st-theme-node.h" */

GType st_side_get_type (void);
#define ST_TYPE_SIDE (st_side_get_type())

GType st_corner_get_type (void);
#define ST_TYPE_CORNER (st_corner_get_type())

GType st_text_decoration_get_type (void);
#define ST_TYPE_TEXT_DECORATION (st_text_decoration_get_type())

GType st_text_align_get_type (void);
#define ST_TYPE_TEXT_ALIGN (st_text_align_get_type())

GType st_gradient_type_get_type (void);
#define ST_TYPE_GRADIENT_TYPE (st_gradient_type_get_type())

GType st_icon_style_get_type (void);
#define ST_TYPE_ICON_STYLE (st_icon_style_get_type())

/* enumerations from "st-types.h" */

GType st_background_size_get_type (void);
#define ST_TYPE_BACKGROUND_SIZE (st_background_size_get_type())

/* enumerations from "st-widget.h" */

GType st_direction_type_get_type (void);
#define ST_TYPE_DIRECTION_TYPE (st_direction_type_get_type())

G_END_DECLS

/* Generated data ends here */

