/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __SWITCHEROO_CONTROL_H__
#define __SWITCHEROO_CONTROL_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for net.hadess.SwitcherooControl */

#define SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL (shell_net_hadess_switcheroo_control_get_type ())
#define SHELL_NET_HADESS_SWITCHEROO_CONTROL(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL, ShellNetHadessSwitcherooControl))
#define SHELL_IS_NET_HADESS_SWITCHEROO_CONTROL(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL))
#define SHELL_NET_HADESS_SWITCHEROO_CONTROL_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL, ShellNetHadessSwitcherooControlIface))

struct _ShellNetHadessSwitcherooControl;
typedef struct _ShellNetHadessSwitcherooControl ShellNetHadessSwitcherooControl;
typedef struct _ShellNetHadessSwitcherooControlIface ShellNetHadessSwitcherooControlIface;

struct _ShellNetHadessSwitcherooControlIface
{
  GTypeInterface parent_iface;

  GVariant * (*get_gpus) (ShellNetHadessSwitcherooControl *object);

  gboolean  (*get_has_dual_gpu) (ShellNetHadessSwitcherooControl *object);

  guint  (*get_num_gpus) (ShellNetHadessSwitcherooControl *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ShellNetHadessSwitcherooControl, g_object_unref)
#endif

GType shell_net_hadess_switcheroo_control_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *shell_net_hadess_switcheroo_control_interface_info (void);
guint shell_net_hadess_switcheroo_control_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
gboolean shell_net_hadess_switcheroo_control_get_has_dual_gpu (ShellNetHadessSwitcherooControl *object);
void shell_net_hadess_switcheroo_control_set_has_dual_gpu (ShellNetHadessSwitcherooControl *object, gboolean value);

guint shell_net_hadess_switcheroo_control_get_num_gpus (ShellNetHadessSwitcherooControl *object);
void shell_net_hadess_switcheroo_control_set_num_gpus (ShellNetHadessSwitcherooControl *object, guint value);

GVariant *shell_net_hadess_switcheroo_control_get_gpus (ShellNetHadessSwitcherooControl *object);
GVariant *shell_net_hadess_switcheroo_control_dup_gpus (ShellNetHadessSwitcherooControl *object);
void shell_net_hadess_switcheroo_control_set_gpus (ShellNetHadessSwitcherooControl *object, GVariant *value);


/* ---- */

#define SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_PROXY (shell_net_hadess_switcheroo_control_proxy_get_type ())
#define SHELL_NET_HADESS_SWITCHEROO_CONTROL_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_PROXY, ShellNetHadessSwitcherooControlProxy))
#define SHELL_NET_HADESS_SWITCHEROO_CONTROL_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_PROXY, ShellNetHadessSwitcherooControlProxyClass))
#define SHELL_NET_HADESS_SWITCHEROO_CONTROL_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_PROXY, ShellNetHadessSwitcherooControlProxyClass))
#define SHELL_IS_NET_HADESS_SWITCHEROO_CONTROL_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_PROXY))
#define SHELL_IS_NET_HADESS_SWITCHEROO_CONTROL_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_PROXY))

typedef struct _ShellNetHadessSwitcherooControlProxy ShellNetHadessSwitcherooControlProxy;
typedef struct _ShellNetHadessSwitcherooControlProxyClass ShellNetHadessSwitcherooControlProxyClass;
typedef struct _ShellNetHadessSwitcherooControlProxyPrivate ShellNetHadessSwitcherooControlProxyPrivate;

struct _ShellNetHadessSwitcherooControlProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  ShellNetHadessSwitcherooControlProxyPrivate *priv;
};

struct _ShellNetHadessSwitcherooControlProxyClass
{
  GDBusProxyClass parent_class;
};

GType shell_net_hadess_switcheroo_control_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ShellNetHadessSwitcherooControlProxy, g_object_unref)
#endif

void shell_net_hadess_switcheroo_control_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
ShellNetHadessSwitcherooControl *shell_net_hadess_switcheroo_control_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
ShellNetHadessSwitcherooControl *shell_net_hadess_switcheroo_control_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void shell_net_hadess_switcheroo_control_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
ShellNetHadessSwitcherooControl *shell_net_hadess_switcheroo_control_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
ShellNetHadessSwitcherooControl *shell_net_hadess_switcheroo_control_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_SKELETON (shell_net_hadess_switcheroo_control_skeleton_get_type ())
#define SHELL_NET_HADESS_SWITCHEROO_CONTROL_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_SKELETON, ShellNetHadessSwitcherooControlSkeleton))
#define SHELL_NET_HADESS_SWITCHEROO_CONTROL_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_SKELETON, ShellNetHadessSwitcherooControlSkeletonClass))
#define SHELL_NET_HADESS_SWITCHEROO_CONTROL_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_SKELETON, ShellNetHadessSwitcherooControlSkeletonClass))
#define SHELL_IS_NET_HADESS_SWITCHEROO_CONTROL_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_SKELETON))
#define SHELL_IS_NET_HADESS_SWITCHEROO_CONTROL_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SHELL_TYPE_NET_HADESS_SWITCHEROO_CONTROL_SKELETON))

typedef struct _ShellNetHadessSwitcherooControlSkeleton ShellNetHadessSwitcherooControlSkeleton;
typedef struct _ShellNetHadessSwitcherooControlSkeletonClass ShellNetHadessSwitcherooControlSkeletonClass;
typedef struct _ShellNetHadessSwitcherooControlSkeletonPrivate ShellNetHadessSwitcherooControlSkeletonPrivate;

struct _ShellNetHadessSwitcherooControlSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  ShellNetHadessSwitcherooControlSkeletonPrivate *priv;
};

struct _ShellNetHadessSwitcherooControlSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType shell_net_hadess_switcheroo_control_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ShellNetHadessSwitcherooControlSkeleton, g_object_unref)
#endif

ShellNetHadessSwitcherooControl *shell_net_hadess_switcheroo_control_skeleton_new (void);


G_END_DECLS

#endif /* __SWITCHEROO_CONTROL_H__ */
