
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "shell-action-modes.h" */

GType shell_action_mode_get_type (void);
#define SHELL_TYPE_ACTION_MODE (shell_action_mode_get_type())

/* enumerations from "shell-app.h" */

GType shell_app_state_get_type (void);
#define SHELL_TYPE_APP_STATE (shell_app_state_get_type())

GType shell_app_launch_gpu_get_type (void);
#define SHELL_TYPE_APP_LAUNCH_GPU (shell_app_launch_gpu_get_type())

/* enumerations from "shell-blur-effect.h" */

GType shell_blur_mode_get_type (void);
#define SHELL_TYPE_BLUR_MODE (shell_blur_mode_get_type())

/* enumerations from "shell-network-agent.h" */

GType shell_network_agent_response_get_type (void);
#define SHELL_TYPE_NETWORK_AGENT_RESPONSE (shell_network_agent_response_get_type())

G_END_DECLS

/* Generated data ends here */

