/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */

#pragma once

#include <glib.h>

G_GNUC_UNUSED static const gchar *docs_mimetypes[] = {
  "application/vnd.oasis.opendocument.text",
  "application/vnd.oasis.opendocument.presentation",
  "application/vnd.oasis.opendocument.spreadsheet",
  "application/msword",
  "application/vnd.ms-excel",
  "application/vnd.ms-powerpoint",
  "application/rtf",
  "application/pdf",
  "application/x-bzpdf",
  "application/x-gzpdf",
  "application/x-xzpdf",
  "application/postscript",
  "application/x-bzpostscript",
  "application/x-gzpostscript",
  "image/x-eps",
  "image/x-bzeps",
  "image/x-gzeps",
  "application/x-dvi",
  "application/x-bzdvi",
  "application/x-gzdvi",
  "image/vnd.djvu",
  "application/x-cbr",
  "application/x-cbz",
  "application/x-cb7",
  "application/x-cbt",
  NULL
};

G_GNUC_UNUSED static const gchar *video_mimetypes[] = {
  "application/mxf",
  "application/ogg",
  "application/ram",
  "application/sdp",
  "application/vnd.ms-wpl",
  "application/vnd.rn-realmedia",
  "application/x-extension-m4a",
  "application/x-extension-mp4",
  "application/x-flash-video",
  "application/x-matroska",
  "application/x-netshow-channel",
  "application/x-ogg",
  "application/x-quicktimeplayer",
  "application/x-shorten",
  "image/vnd.rn-realpix",
  "image/x-pict",
  "misc/ultravox",
  "text/x-google-video-pointer",
  "video/3gpp",
  "video/dv",
  "video/fli",
  "video/flv",
  "video/mp2t",
  "video/mp4",
  "video/mp4v-es",
  "video/mpeg",
  "video/msvideo",
  "video/ogg",
  "video/quicktime",
  "video/vivo",
  "video/vnd.divx",
  "video/vnd.rn-realvideo",
  "video/vnd.vivo",
  "video/webm",
  "video/x-anim",
  "video/x-avi",
  "video/x-flc",
  "video/x-fli",
  "video/x-flic",
  "video/x-flv",
  "video/x-m4v",
  "video/x-matroska",
  "video/x-mpeg",
  "video/x-ms-asf",
  "video/x-ms-asx",
  "video/x-msvideo",
  "video/x-ms-wm",
  "video/x-ms-wmv",
  "video/x-ms-wmx",
  "video/x-ms-wvx",
  "video/x-nsv",
  "video/x-ogm+ogg",
  "video/x-theora+ogg",
  "video/x-totem-stream",
  NULL
};

G_GNUC_UNUSED static const gchar *audio_mimetypes[] = {
  "audio/3gpp",
  "audio/ac3",
  "audio/AMR",
  "audio/AMR-WB",
  "audio/basic",
  "audio/flac",
  "audio/midi",
  "audio/mp2",
  "audio/mp4",
  "audio/mpeg",
  "audio/ogg",
  "audio/prs.sid",
  "audio/vnd.rn-realaudio",
  "audio/x-aiff",
  "audio/x-ape",
  "audio/x-flac",
  "audio/x-gsm",
  "audio/x-it",
  "audio/x-m4a",
  "audio/x-matroska",
  "audio/x-mod",
  "audio/x-mp3",
  "audio/x-mpeg",
  "audio/x-ms-asf",
  "audio/x-ms-asx",
  "audio/x-ms-wax",
  "audio/x-ms-wma",
  "audio/x-musepack",
  "audio/x-pn-aiff",
  "audio/x-pn-au",
  "audio/x-pn-wav",
  "audio/x-pn-windows-acm",
  "audio/x-realaudio",
  "audio/x-real-audio",
  "audio/x-sbc",
  "audio/x-speex",
  "audio/x-tta",
  "audio/x-wav",
  "audio/x-wavpack",
  "audio/x-vorbis",
  "audio/x-vorbis+ogg",
  "audio/x-xm",
  NULL
};
