
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "shell-enum-types.h"
#include "shell-app.h"
#include "shell-app-system.h"
#include "shell-app-usage.h"
#include "shell-blur-effect.h"
#include "shell-camera-monitor.h"
#include "shell-edge-drag-gesture.h"
#include "shell-glsl-effect.h"
#include "shell-global.h"
#include "shell-invert-lightness-effect.h"
#include "shell-action-modes.h"
#include "shell-mount-operation.h"
#include "shell-perf-log.h"
#include "shell-screenshot.h"
#include "shell-square-bin.h"
#include "shell-stack.h"
#include "shell-time-change-source.h"
#include "shell-util.h"
#include "shell-window-preview.h"
#include "shell-window-preview-layout.h"
#include "shell-window-tracker.h"
#include "shell-wm.h"
#include "shell-workspace-background.h"
#include "shell-tray-icon.h"
#include "shell-tray-manager.h"
#include "shell-network-agent.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "shell-action-modes.h" */

GType
shell_action_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (SHELL_ACTION_MODE_NONE), "SHELL_ACTION_MODE_NONE", "none" },
        { C_FLAGS (SHELL_ACTION_MODE_NORMAL), "SHELL_ACTION_MODE_NORMAL", "normal" },
        { C_FLAGS (SHELL_ACTION_MODE_OVERVIEW), "SHELL_ACTION_MODE_OVERVIEW", "overview" },
        { C_FLAGS (SHELL_ACTION_MODE_LOCK_SCREEN), "SHELL_ACTION_MODE_LOCK_SCREEN", "lock-screen" },
        { C_FLAGS (SHELL_ACTION_MODE_UNLOCK_SCREEN), "SHELL_ACTION_MODE_UNLOCK_SCREEN", "unlock-screen" },
        { C_FLAGS (SHELL_ACTION_MODE_LOGIN_SCREEN), "SHELL_ACTION_MODE_LOGIN_SCREEN", "login-screen" },
        { C_FLAGS (SHELL_ACTION_MODE_SYSTEM_MODAL), "SHELL_ACTION_MODE_SYSTEM_MODAL", "system-modal" },
        { C_FLAGS (SHELL_ACTION_MODE_LOOKING_GLASS), "SHELL_ACTION_MODE_LOOKING_GLASS", "looking-glass" },
        { C_FLAGS (SHELL_ACTION_MODE_POPUP), "SHELL_ACTION_MODE_POPUP", "popup" },
        { C_FLAGS (SHELL_ACTION_MODE_ALL), "SHELL_ACTION_MODE_ALL", "all" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("ShellActionMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "shell-app.h" */

GType
shell_app_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (SHELL_APP_STATE_STOPPED), "SHELL_APP_STATE_STOPPED", "stopped" },
        { C_ENUM (SHELL_APP_STATE_STARTING), "SHELL_APP_STATE_STARTING", "starting" },
        { C_ENUM (SHELL_APP_STATE_RUNNING), "SHELL_APP_STATE_RUNNING", "running" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ShellAppState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
shell_app_launch_gpu_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (SHELL_APP_LAUNCH_GPU_APP_PREF), "SHELL_APP_LAUNCH_GPU_APP_PREF", "app-pref" },
        { C_ENUM (SHELL_APP_LAUNCH_GPU_DISCRETE), "SHELL_APP_LAUNCH_GPU_DISCRETE", "discrete" },
        { C_ENUM (SHELL_APP_LAUNCH_GPU_DEFAULT), "SHELL_APP_LAUNCH_GPU_DEFAULT", "default" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ShellAppLaunchGpu"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "shell-blur-effect.h" */

GType
shell_blur_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (SHELL_BLUR_MODE_ACTOR), "SHELL_BLUR_MODE_ACTOR", "actor" },
        { C_ENUM (SHELL_BLUR_MODE_BACKGROUND), "SHELL_BLUR_MODE_BACKGROUND", "background" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ShellBlurMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "shell-network-agent.h" */

GType
shell_network_agent_response_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (SHELL_NETWORK_AGENT_CONFIRMED), "SHELL_NETWORK_AGENT_CONFIRMED", "confirmed" },
        { C_ENUM (SHELL_NETWORK_AGENT_USER_CANCELED), "SHELL_NETWORK_AGENT_USER_CANCELED", "user-canceled" },
        { C_ENUM (SHELL_NETWORK_AGENT_INTERNAL_ERROR), "SHELL_NETWORK_AGENT_INTERNAL_ERROR", "internal-error" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("ShellNetworkAgentResponse"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

