
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "st-enum-types.h"
#include "st-adjustment.h"
#include "st-bin.h"
#include "st-border-image.h"
#include "st-box-layout.h"
#include "st-button.h"
#include "st-clipboard.h"
#include "st-dnd-start-gesture.h"
#include "st-drawing-area.h"
#include "st-entry.h"
#include "st-focus-manager.h"
#include "st-generic-accessible.h"
#include "st-icon.h"
#include "st-icon-colors.h"
#include "st-icon-theme.h"
#include "st-image-content.h"
#include "st-label.h"
#include "st-password-entry.h"
#include "st-scrollable.h"
#include "st-scroll-bar.h"
#include "st-scroll-view.h"
#include "st-scroll-view-fade.h"
#include "st-settings.h"
#include "st-shadow.h"
#include "st-spinner-content.h"
#include "st-texture-cache.h"
#include "st-theme.h"
#include "st-theme-context.h"
#include "st-theme-node.h"
#include "st-types.h"
#include "st-viewport.h"
#include "st-widget.h"
#include "st-widget-accessible.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "st-button.h" */

GType
st_button_mask_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (ST_BUTTON_ONE), "ST_BUTTON_ONE", "one" },
        { C_FLAGS (ST_BUTTON_TWO), "ST_BUTTON_TWO", "two" },
        { C_FLAGS (ST_BUTTON_THREE), "ST_BUTTON_THREE", "three" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("StButtonMask"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "st-clipboard.h" */

GType
st_clipboard_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_CLIPBOARD_TYPE_PRIMARY), "ST_CLIPBOARD_TYPE_PRIMARY", "primary" },
        { C_ENUM (ST_CLIPBOARD_TYPE_CLIPBOARD), "ST_CLIPBOARD_TYPE_CLIPBOARD", "clipboard" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StClipboardType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "st-icon-theme.h" */

GType
st_icon_lookup_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (ST_ICON_LOOKUP_NO_SVG), "ST_ICON_LOOKUP_NO_SVG", "no-svg" },
        { C_FLAGS (ST_ICON_LOOKUP_FORCE_SVG), "ST_ICON_LOOKUP_FORCE_SVG", "force-svg" },
        { C_FLAGS (ST_ICON_LOOKUP_GENERIC_FALLBACK), "ST_ICON_LOOKUP_GENERIC_FALLBACK", "generic-fallback" },
        { C_FLAGS (ST_ICON_LOOKUP_FORCE_SIZE), "ST_ICON_LOOKUP_FORCE_SIZE", "force-size" },
        { C_FLAGS (ST_ICON_LOOKUP_FORCE_REGULAR), "ST_ICON_LOOKUP_FORCE_REGULAR", "force-regular" },
        { C_FLAGS (ST_ICON_LOOKUP_FORCE_SYMBOLIC), "ST_ICON_LOOKUP_FORCE_SYMBOLIC", "force-symbolic" },
        { C_FLAGS (ST_ICON_LOOKUP_DIR_LTR), "ST_ICON_LOOKUP_DIR_LTR", "dir-ltr" },
        { C_FLAGS (ST_ICON_LOOKUP_DIR_RTL), "ST_ICON_LOOKUP_DIR_RTL", "dir-rtl" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("StIconLookupFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
st_icon_theme_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_ICON_THEME_NOT_FOUND), "ST_ICON_THEME_NOT_FOUND", "not-found" },
        { C_ENUM (ST_ICON_THEME_FAILED), "ST_ICON_THEME_FAILED", "failed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StIconThemeError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "st-scroll-view.h" */

GType
st_policy_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_POLICY_ALWAYS), "ST_POLICY_ALWAYS", "always" },
        { C_ENUM (ST_POLICY_AUTOMATIC), "ST_POLICY_AUTOMATIC", "automatic" },
        { C_ENUM (ST_POLICY_NEVER), "ST_POLICY_NEVER", "never" },
        { C_ENUM (ST_POLICY_EXTERNAL), "ST_POLICY_EXTERNAL", "external" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StPolicyType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "st-settings.h" */

GType
st_system_color_scheme_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_SYSTEM_COLOR_SCHEME_DEFAULT), "ST_SYSTEM_COLOR_SCHEME_DEFAULT", "default" },
        { C_ENUM (ST_SYSTEM_COLOR_SCHEME_PREFER_DARK), "ST_SYSTEM_COLOR_SCHEME_PREFER_DARK", "prefer-dark" },
        { C_ENUM (ST_SYSTEM_COLOR_SCHEME_PREFER_LIGHT), "ST_SYSTEM_COLOR_SCHEME_PREFER_LIGHT", "prefer-light" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StSystemColorScheme"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
st_system_accent_color_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_SYSTEM_ACCENT_COLOR_BLUE), "ST_SYSTEM_ACCENT_COLOR_BLUE", "blue" },
        { C_ENUM (ST_SYSTEM_ACCENT_COLOR_TEAL), "ST_SYSTEM_ACCENT_COLOR_TEAL", "teal" },
        { C_ENUM (ST_SYSTEM_ACCENT_COLOR_GREEN), "ST_SYSTEM_ACCENT_COLOR_GREEN", "green" },
        { C_ENUM (ST_SYSTEM_ACCENT_COLOR_YELLOW), "ST_SYSTEM_ACCENT_COLOR_YELLOW", "yellow" },
        { C_ENUM (ST_SYSTEM_ACCENT_COLOR_ORANGE), "ST_SYSTEM_ACCENT_COLOR_ORANGE", "orange" },
        { C_ENUM (ST_SYSTEM_ACCENT_COLOR_RED), "ST_SYSTEM_ACCENT_COLOR_RED", "red" },
        { C_ENUM (ST_SYSTEM_ACCENT_COLOR_PINK), "ST_SYSTEM_ACCENT_COLOR_PINK", "pink" },
        { C_ENUM (ST_SYSTEM_ACCENT_COLOR_PURPLE), "ST_SYSTEM_ACCENT_COLOR_PURPLE", "purple" },
        { C_ENUM (ST_SYSTEM_ACCENT_COLOR_SLATE), "ST_SYSTEM_ACCENT_COLOR_SLATE", "slate" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StSystemAccentColor"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "st-texture-cache.h" */

GType
st_texture_cache_policy_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_TEXTURE_CACHE_POLICY_NONE), "ST_TEXTURE_CACHE_POLICY_NONE", "none" },
        { C_ENUM (ST_TEXTURE_CACHE_POLICY_FOREVER), "ST_TEXTURE_CACHE_POLICY_FOREVER", "forever" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StTextureCachePolicy"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "st-theme-node.h" */

GType
st_side_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_SIDE_TOP), "ST_SIDE_TOP", "top" },
        { C_ENUM (ST_SIDE_RIGHT), "ST_SIDE_RIGHT", "right" },
        { C_ENUM (ST_SIDE_BOTTOM), "ST_SIDE_BOTTOM", "bottom" },
        { C_ENUM (ST_SIDE_LEFT), "ST_SIDE_LEFT", "left" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StSide"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
st_corner_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_CORNER_TOPLEFT), "ST_CORNER_TOPLEFT", "topleft" },
        { C_ENUM (ST_CORNER_TOPRIGHT), "ST_CORNER_TOPRIGHT", "topright" },
        { C_ENUM (ST_CORNER_BOTTOMRIGHT), "ST_CORNER_BOTTOMRIGHT", "bottomright" },
        { C_ENUM (ST_CORNER_BOTTOMLEFT), "ST_CORNER_BOTTOMLEFT", "bottomleft" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StCorner"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
st_text_decoration_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (ST_TEXT_DECORATION_UNDERLINE), "ST_TEXT_DECORATION_UNDERLINE", "underline" },
        { C_FLAGS (ST_TEXT_DECORATION_OVERLINE), "ST_TEXT_DECORATION_OVERLINE", "overline" },
        { C_FLAGS (ST_TEXT_DECORATION_LINE_THROUGH), "ST_TEXT_DECORATION_LINE_THROUGH", "line-through" },
        { C_FLAGS (ST_TEXT_DECORATION_BLINK), "ST_TEXT_DECORATION_BLINK", "blink" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("StTextDecoration"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
st_text_align_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_TEXT_ALIGN_LEFT), "ST_TEXT_ALIGN_LEFT", "left" },
        { C_ENUM (ST_TEXT_ALIGN_CENTER), "ST_TEXT_ALIGN_CENTER", "center" },
        { C_ENUM (ST_TEXT_ALIGN_RIGHT), "ST_TEXT_ALIGN_RIGHT", "right" },
        { C_ENUM (ST_TEXT_ALIGN_JUSTIFY), "ST_TEXT_ALIGN_JUSTIFY", "justify" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StTextAlign"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
st_gradient_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_GRADIENT_NONE), "ST_GRADIENT_NONE", "none" },
        { C_ENUM (ST_GRADIENT_VERTICAL), "ST_GRADIENT_VERTICAL", "vertical" },
        { C_ENUM (ST_GRADIENT_HORIZONTAL), "ST_GRADIENT_HORIZONTAL", "horizontal" },
        { C_ENUM (ST_GRADIENT_RADIAL), "ST_GRADIENT_RADIAL", "radial" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StGradientType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
st_icon_style_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_ICON_STYLE_REQUESTED), "ST_ICON_STYLE_REQUESTED", "requested" },
        { C_ENUM (ST_ICON_STYLE_REGULAR), "ST_ICON_STYLE_REGULAR", "regular" },
        { C_ENUM (ST_ICON_STYLE_SYMBOLIC), "ST_ICON_STYLE_SYMBOLIC", "symbolic" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StIconStyle"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "st-types.h" */

GType
st_background_size_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_BACKGROUND_SIZE_AUTO), "ST_BACKGROUND_SIZE_AUTO", "auto" },
        { C_ENUM (ST_BACKGROUND_SIZE_CONTAIN), "ST_BACKGROUND_SIZE_CONTAIN", "contain" },
        { C_ENUM (ST_BACKGROUND_SIZE_COVER), "ST_BACKGROUND_SIZE_COVER", "cover" },
        { C_ENUM (ST_BACKGROUND_SIZE_FIXED), "ST_BACKGROUND_SIZE_FIXED", "fixed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StBackgroundSize"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "st-widget.h" */

GType
st_direction_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (ST_DIR_TAB_FORWARD), "ST_DIR_TAB_FORWARD", "tab-forward" },
        { C_ENUM (ST_DIR_TAB_BACKWARD), "ST_DIR_TAB_BACKWARD", "tab-backward" },
        { C_ENUM (ST_DIR_UP), "ST_DIR_UP", "up" },
        { C_ENUM (ST_DIR_DOWN), "ST_DIR_DOWN", "down" },
        { C_ENUM (ST_DIR_LEFT), "ST_DIR_LEFT", "left" },
        { C_ENUM (ST_DIR_RIGHT), "ST_DIR_RIGHT", "right" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("StDirectionType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

