/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __ORG_GTK_APPLICATION_H__
#define __ORG_GTK_APPLICATION_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gtk.Application */

#define SHELL_TYPE_ORG_GTK_APPLICATION (shell_org_gtk_application_get_type ())
#define SHELL_ORG_GTK_APPLICATION(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SHELL_TYPE_ORG_GTK_APPLICATION, ShellOrgGtkApplication))
#define SHELL_IS_ORG_GTK_APPLICATION(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SHELL_TYPE_ORG_GTK_APPLICATION))
#define SHELL_ORG_GTK_APPLICATION_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), SHELL_TYPE_ORG_GTK_APPLICATION, ShellOrgGtkApplicationIface))

struct _ShellOrgGtkApplication;
typedef struct _ShellOrgGtkApplication ShellOrgGtkApplication;
typedef struct _ShellOrgGtkApplicationIface ShellOrgGtkApplicationIface;

struct _ShellOrgGtkApplicationIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_activate) (
    ShellOrgGtkApplication *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_platform_data);

  gboolean (*handle_command_line) (
    ShellOrgGtkApplication *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_path,
    const gchar *const *arg_arguments,
    GVariant *arg_platform_data);

  gboolean (*handle_open) (
    ShellOrgGtkApplication *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *arg_uris,
    const gchar *arg_hint,
    GVariant *arg_platform_data);

  gboolean  (*get_busy) (ShellOrgGtkApplication *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ShellOrgGtkApplication, g_object_unref)
#endif

GType shell_org_gtk_application_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *shell_org_gtk_application_interface_info (void);
guint shell_org_gtk_application_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void shell_org_gtk_application_complete_activate (
    ShellOrgGtkApplication *object,
    GDBusMethodInvocation *invocation);

void shell_org_gtk_application_complete_open (
    ShellOrgGtkApplication *object,
    GDBusMethodInvocation *invocation);

void shell_org_gtk_application_complete_command_line (
    ShellOrgGtkApplication *object,
    GDBusMethodInvocation *invocation,
    gint exit_status);



/* D-Bus method calls: */
void shell_org_gtk_application_call_activate (
    ShellOrgGtkApplication *proxy,
    GVariant *arg_platform_data,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean shell_org_gtk_application_call_activate_finish (
    ShellOrgGtkApplication *proxy,
    GAsyncResult *res,
    GError **error);

gboolean shell_org_gtk_application_call_activate_sync (
    ShellOrgGtkApplication *proxy,
    GVariant *arg_platform_data,
    GCancellable *cancellable,
    GError **error);

void shell_org_gtk_application_call_open (
    ShellOrgGtkApplication *proxy,
    const gchar *const *arg_uris,
    const gchar *arg_hint,
    GVariant *arg_platform_data,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean shell_org_gtk_application_call_open_finish (
    ShellOrgGtkApplication *proxy,
    GAsyncResult *res,
    GError **error);

gboolean shell_org_gtk_application_call_open_sync (
    ShellOrgGtkApplication *proxy,
    const gchar *const *arg_uris,
    const gchar *arg_hint,
    GVariant *arg_platform_data,
    GCancellable *cancellable,
    GError **error);

void shell_org_gtk_application_call_command_line (
    ShellOrgGtkApplication *proxy,
    const gchar *arg_path,
    const gchar *const *arg_arguments,
    GVariant *arg_platform_data,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean shell_org_gtk_application_call_command_line_finish (
    ShellOrgGtkApplication *proxy,
    gint* out_exit_status,
    GAsyncResult *res,
    GError **error);

gboolean shell_org_gtk_application_call_command_line_sync (
    ShellOrgGtkApplication *proxy,
    const gchar *arg_path,
    const gchar *const *arg_arguments,
    GVariant *arg_platform_data,
    gint* out_exit_status,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean shell_org_gtk_application_get_busy (ShellOrgGtkApplication *object);
void shell_org_gtk_application_set_busy (ShellOrgGtkApplication *object, gboolean value);


/* ---- */

#define SHELL_TYPE_ORG_GTK_APPLICATION_PROXY (shell_org_gtk_application_proxy_get_type ())
#define SHELL_ORG_GTK_APPLICATION_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SHELL_TYPE_ORG_GTK_APPLICATION_PROXY, ShellOrgGtkApplicationProxy))
#define SHELL_ORG_GTK_APPLICATION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SHELL_TYPE_ORG_GTK_APPLICATION_PROXY, ShellOrgGtkApplicationProxyClass))
#define SHELL_ORG_GTK_APPLICATION_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SHELL_TYPE_ORG_GTK_APPLICATION_PROXY, ShellOrgGtkApplicationProxyClass))
#define SHELL_IS_ORG_GTK_APPLICATION_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SHELL_TYPE_ORG_GTK_APPLICATION_PROXY))
#define SHELL_IS_ORG_GTK_APPLICATION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SHELL_TYPE_ORG_GTK_APPLICATION_PROXY))

typedef struct _ShellOrgGtkApplicationProxy ShellOrgGtkApplicationProxy;
typedef struct _ShellOrgGtkApplicationProxyClass ShellOrgGtkApplicationProxyClass;
typedef struct _ShellOrgGtkApplicationProxyPrivate ShellOrgGtkApplicationProxyPrivate;

struct _ShellOrgGtkApplicationProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  ShellOrgGtkApplicationProxyPrivate *priv;
};

struct _ShellOrgGtkApplicationProxyClass
{
  GDBusProxyClass parent_class;
};

GType shell_org_gtk_application_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ShellOrgGtkApplicationProxy, g_object_unref)
#endif

void shell_org_gtk_application_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
ShellOrgGtkApplication *shell_org_gtk_application_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
ShellOrgGtkApplication *shell_org_gtk_application_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void shell_org_gtk_application_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
ShellOrgGtkApplication *shell_org_gtk_application_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
ShellOrgGtkApplication *shell_org_gtk_application_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define SHELL_TYPE_ORG_GTK_APPLICATION_SKELETON (shell_org_gtk_application_skeleton_get_type ())
#define SHELL_ORG_GTK_APPLICATION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SHELL_TYPE_ORG_GTK_APPLICATION_SKELETON, ShellOrgGtkApplicationSkeleton))
#define SHELL_ORG_GTK_APPLICATION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SHELL_TYPE_ORG_GTK_APPLICATION_SKELETON, ShellOrgGtkApplicationSkeletonClass))
#define SHELL_ORG_GTK_APPLICATION_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), SHELL_TYPE_ORG_GTK_APPLICATION_SKELETON, ShellOrgGtkApplicationSkeletonClass))
#define SHELL_IS_ORG_GTK_APPLICATION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SHELL_TYPE_ORG_GTK_APPLICATION_SKELETON))
#define SHELL_IS_ORG_GTK_APPLICATION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SHELL_TYPE_ORG_GTK_APPLICATION_SKELETON))

typedef struct _ShellOrgGtkApplicationSkeleton ShellOrgGtkApplicationSkeleton;
typedef struct _ShellOrgGtkApplicationSkeletonClass ShellOrgGtkApplicationSkeletonClass;
typedef struct _ShellOrgGtkApplicationSkeletonPrivate ShellOrgGtkApplicationSkeletonPrivate;

struct _ShellOrgGtkApplicationSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  ShellOrgGtkApplicationSkeletonPrivate *priv;
};

struct _ShellOrgGtkApplicationSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType shell_org_gtk_application_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ShellOrgGtkApplicationSkeleton, g_object_unref)
#endif

ShellOrgGtkApplication *shell_org_gtk_application_skeleton_new (void);


G_END_DECLS

#endif /* __ORG_GTK_APPLICATION_H__ */
